==> cx88-1.0.2.tar.gz: cx88-1.0.2/tests/hd3k_analog_kbuf.cpp <==
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include "cx88_video_ioctl.h"
#include "tuner_devnode_device.h"
#include "dtt7612.h"
#include "tda9887.h"

void capture(int fd, void *bufspace, cx88_video_format &fmt, cx88_video_config &config)
{
   cx88_video_capture capture;
   capture.in_input_source = CX88_VIDEO_INPUT_MUX1;
   capture.in_input_type = CX88_VIDEO_INPUT_COMPOSITE;
   capture.in_input_options = 0;
   capture.in_lock_timeout_ms = 1000;
   capture.in_lock_delay_ms = 100;
   capture.in_no_frame_intrs = 0;
   printf("Beginning capture\n");
   int error = ioctl(fd, CX88_VIDEO_CAPTURE, &capture);
   if (error)
   {
      printf("Error during capture ioctl\n");
      return;
   }
   char filename[32];
   for (int i = 0; i < config.in_num_frames; ++i)
   {
      cx88_video_wait wait;
      wait.in_timeout_ms = 0;
      if ((i == 0) || (wait.channels[CX88_VIDEO_CHANNEL_Y].out_even_frame_available <= i))
      {
         wait.in_channel_mask = CX88_VIDEO_FIELD_EVEN(CX88_VIDEO_CHANNEL_Y);
         error = ioctl(fd, CX88_VIDEO_WAIT, &wait);
         if (error)
         {
            printf("Error during wait ioctl\n");
            ioctl(fd, CX88_VIDEO_STOP);
            return;
         }
         if ((wait.out_status & CX88_VIDEO_BUS_ERROR) == CX88_VIDEO_BUS_ERROR)
         {
            printf("Fatal error: PCI bus error\n");
         }
         if ((wait.out_status & CX88_VIDEO_SYNC_ERROR) == CX88_VIDEO_SYNC_ERROR)
         {
            printf("Fatal error[%d]: DMA sync error\n", i);  
         }
         if ((wait.out_status & CX88_VIDEO_DMA_PROGRAM) == CX88_VIDEO_DMA_PROGRAM)
         {
            printf("Fatal error[%d]: DMA program error\n", i);
         }
         if ((wait.out_status & CX88_VIDEO_WAIT_ABORTED) == CX88_VIDEO_WAIT_ABORTED)
         {
            printf("Fatal error: Wait aborted\n");
         }
         if ((wait.out_status & CX88_VIDEO_CAP_TIMEOUT) == CX88_VIDEO_CAP_TIMEOUT)
         {
            printf("Nonfatal error: Capture timed out\n");
         }
         if (wait.out_status & CX88_VIDEO_CAP_FATAL)
         {
            ioctl(fd, CX88_VIDEO_STOP);
            return;
         }
      }
      /*if ((wait.channels[CX88_VIDEO_CHANNEL_Y].out_even_frame_available <= i) ||
          (wait.channels[CX88_VIDEO_CHANNEL_Y].out_odd_frame_available <= i))
      {
         printf("No frame available %d %d\n",
            wait.channels[CX88_VIDEO_CHANNEL_Y].out_even_frame_available,
            wait.channels[CX88_VIDEO_CHANNEL_Y].out_odd_frame_available);
         ioctl(fd, CX88_VIDEO_STOP);
         return;
      }*/
      printf("Y frames: %d, %d\n", 
         wait.channels[CX88_VIDEO_CHANNEL_Y].out_odd_frame_available,
         wait.channels[CX88_VIDEO_CHANNEL_Y].out_even_frame_available);
      printf("VBI frames: %d, %d\n",
         wait.channels[CX88_VIDEO_CHANNEL_VBI].out_odd_frame_available,
         wait.channels[CX88_VIDEO_CHANNEL_VBI].out_even_frame_available);
      printf("U frames: %d, %d\n",
         wait.channels[CX88_VIDEO_CHANNEL_U].out_odd_frame_available,
         wait.channels[CX88_VIDEO_CHANNEL_U].out_even_frame_available);
      printf("V frames: %d, %d\n",
         wait.channels[CX88_VIDEO_CHANNEL_V].out_odd_frame_available,
         wait.channels[CX88_VIDEO_CHANNEL_V].out_even_frame_available);
      sprintf(filename, "out%d.ppm", i);
      FILE *outfile = fopen(filename, "w");
      if (outfile == NULL)
      {
         printf("Error opening video output file\n");
         ioctl(fd, CX88_VIDEO_STOP);
         return;  
      }
      fprintf(outfile, "P6\n%d %d\n255\n", fmt.out_geoms[CX88_VIDEO_CHANNEL_Y].line_size / 3, fmt.out_geoms[CX88_VIDEO_CHANNEL_Y].num_lines);
      uint32_t bufsize = fmt.out_geoms[CX88_VIDEO_CHANNEL_Y].line_size * fmt.out_geoms[CX88_VIDEO_CHANNEL_Y].num_lines;
      uint8_t *ybuf = reinterpret_cast <uint8_t*> (bufspace) + 
         (i * config.bufconfig[CX88_VIDEO_CHANNEL_Y].u.out_kernelbuf.out_buffer_size);
      uint8_t *ubuf = reinterpret_cast <uint8_t*> (bufspace) + 
         config.bufconfig[CX88_VIDEO_CHANNEL_Y].u.out_kernelbuf.out_mmap_size +
         (i * config.bufconfig[CX88_VIDEO_CHANNEL_U].u.out_kernelbuf.out_buffer_size);
      uint8_t *vbuf = reinterpret_cast <uint8_t*> (bufspace) +
         config.bufconfig[CX88_VIDEO_CHANNEL_Y].u.out_kernelbuf.out_mmap_size +
         config.bufconfig[CX88_VIDEO_CHANNEL_U].u.out_kernelbuf.out_mmap_size +
         (i * config.bufconfig[CX88_VIDEO_CHANNEL_V].u.out_kernelbuf.out_buffer_size);
      /*for (size_t j = 0; j < bufsize; ++j)
      {
         uint8_t pixel[3];
         int8_t u = ubuf[j / 4] - 128;
         int8_t v = vbuf[j / 4] - 128;
         double r = 1.370705 * v + ybuf[j];
         double g = (double)(ybuf[j]) - (0.337633 * u) - (0.698001 * v);
         double b = 1.732446 * u + ybuf[j];
         if (r < 0) r = 0;
         if (g < 0) g = 0;
         if (b < 0) b = 0;
         if (r > 255) r = 255;
         if (g > 255) g = 255;
         if (b > 255) b = 255;
         pixel[0] = (uint8_t) r;
         pixel[1] = (uint8_t) g;
         pixel[2] = (uint8_t) b;
         fwrite(pixel, 3, 1, outfile);
      }*/
      for (size_t j = 0; j < bufsize; j += 3)
      {
         uint8_t pixel[3];
         pixel[0] = ybuf[j + 2];
         pixel[1] = ybuf[j + 1];
         pixel[2] = ybuf[j];
         fwrite(pixel, 3, 1, outfile);
      }
      fclose(outfile);
      if (fmt.in_enable_vbi)
      {
         if ((wait.channels[CX88_VIDEO_CHANNEL_VBI].out_even_frame_available <= i) ||
            (wait.channels[CX88_VIDEO_CHANNEL_VBI].out_odd_frame_available <= i))
         {
            printf("Warning: no VBI frame available\n");
            continue;
         }
         char *vbibuf = reinterpret_cast <char*> (bufspace) + 
            config.bufconfig[CX88_VIDEO_CHANNEL_Y].u.out_kernelbuf.out_mmap_size + 
            config.bufconfig[CX88_VIDEO_CHANNEL_U].u.out_kernelbuf.out_mmap_size +
            config.bufconfig[CX88_VIDEO_CHANNEL_V].u.out_kernelbuf.out_mmap_size +
            (i * config.bufconfig[CX88_VIDEO_CHANNEL_VBI].u.out_kernelbuf.out_buffer_size);
         bufsize = fmt.out_geoms[CX88_VIDEO_CHANNEL_VBI].line_size * fmt.out_geoms[CX88_VIDEO_CHANNEL_VBI].num_lines;
         sprintf(filename, "out%d.vbi", i);
         FILE *vbifile = fopen(filename, "w");
         if (vbifile == NULL)
         {
            printf("Warning: unable to open VBI output file\n");
            continue;
         }
         fwrite(vbibuf, bufsize, 1, vbifile);
         fclose(vbifile);
      }
   }
   ioctl(fd, CX88_VIDEO_STOP);
}

void tune(char *tuner_devnode, int fd, int buffers, void *bufspace, cx88_video_format &fmt, cx88_video_config &config)
{
   int error = 0;
   avb_channel channel;
   channel.format = AVB_FORMAT_NTSC_M;
   channel.frequency_hz = 177000000;
   channel.bandwidth_hz = 6000000;
   tuner_config conf;
   error = conf.load_file("tuner_config");
   if (error)
   {
      printf("Error loading tuner config file\n");
      return;
   }
   tuner_devnode_device tuner_dev(conf, tuner_devnode, error);
   if (error)
   {
      printf("Error creating tuner devnode\n");
      return;
   }
   dtt7612 tuner_driver(conf, tuner_dev);
   error = tuner_driver.set_channel(channel);
   if (error)
   {
      printf("Error setting channel on tuner driver\n");
      return;  
   }
   printf("Ready to start tuner\n");
   error = tuner_driver.start(10000);
   if (error)
   {
      printf("Error starting tuner\n");
      return;
   }
   sleep(1);
   capture(fd, bufspace, fmt, config);
}

int main(void)
{
   cx88_video_format fmt;
   fmt.in_video_format = CX88_VIDEO_FMT_NTSCM;
   fmt.in_pixel_format = CX88_VIDEO_BGR24;
   fmt.in_rows = 480;
   fmt.in_cols = 720;
   fmt.in_x_offset = 0;
   fmt.in_x_size = 720;
   fmt.in_y_offset = 0;
   fmt.in_y_size = 480;
   fmt.in_enable_vbi = 0;
   fmt.in_interlace = 1;
   fmt.in_even_field.swap = 0;
   fmt.in_even_field.enable = 1;
   fmt.in_odd_field.swap = 0;
   fmt.in_odd_field.enable = 1;
   cx88_video_config config;
   uint32_t id;
   config.in_num_frames = 7;
   config.in_one_shot = 0;
   config.bufconfig[CX88_VIDEO_CHANNEL_Y].in_buftype = CX88_VIDEO_BUF_KERNEL;
   config.bufconfig[CX88_VIDEO_CHANNEL_U].in_buftype = CX88_VIDEO_BUF_KERNEL;
   config.bufconfig[CX88_VIDEO_CHANNEL_V].in_buftype = CX88_VIDEO_BUF_KERNEL;
   config.bufconfig[CX88_VIDEO_CHANNEL_VBI].in_buftype = CX88_VIDEO_BUF_KERNEL;
   int fd = open("/dev/cx88video0", O_RDWR);
   int error = ioctl(fd, CX88_VIDEO_CARD_ID, &id);
   if (error)
   {
      printf("Error during card ID ioctl\n");
      return 1;
   }
   printf("Card ID = 0x%x\n", id);
   error = ioctl(fd, CX88_VIDEO_FORMAT, &fmt);
   if (error)
   {  
      printf("Error during format ioctl: %s\n", strerror(error));
      return 1;
   }
   printf("Y Geometry: %d x %d\n", fmt.out_geoms[CX88_VIDEO_CHANNEL_Y].line_size, fmt.out_geoms[CX88_VIDEO_CHANNEL_Y].num_lines);
   printf("U Geometry: %d x %d\n", fmt.out_geoms[CX88_VIDEO_CHANNEL_U].line_size, fmt.out_geoms[CX88_VIDEO_CHANNEL_U].num_lines);
   printf("V Geometry: %d x %d\n", fmt.out_geoms[CX88_VIDEO_CHANNEL_V].line_size, fmt.out_geoms[CX88_VIDEO_CHANNEL_V].num_lines);
   printf("VBI Geometry: %d x %d\n", fmt.out_geoms[CX88_VIDEO_CHANNEL_VBI].line_size, fmt.out_geoms[CX88_VIDEO_CHANNEL_VBI].num_lines);
   error = ioctl(fd, CX88_VIDEO_CONFIG, &config);
   if (error)
   {
      printf("Error during config ioctl: %s\n", strerror(error));
      return 1;
   }   
   cx88_video_i2c_config i2config;
   i2config.in_i2c_addrs[0] = 0x61;
   i2config.in_num_i2c_devices = 1;
   error = ioctl(fd, CX88_VIDEO_I2C_CONFIG, &i2config);
   if (error)
   {
      printf("Error during I2C config ioctl\n");
      return 1;  
   }
   printf("Tuner: %s\n", i2config.out_i2c_devnodes[0]);
   char tuner_devnode[CX88_VIDEO_MAX_DEVNODE_NAME + 5];
   sprintf(tuner_devnode, "/dev/%s", i2config.out_i2c_devnodes[0]);
   void *buf = mmap(NULL,
      config.bufconfig[CX88_VIDEO_CHANNEL_Y].u.out_kernelbuf.out_mmap_size /*+ 
      config.bufconfig[CX88_VIDEO_CHANNEL_U].u.out_kernelbuf.out_mmap_size +
      config.bufconfig[CX88_VIDEO_CHANNEL_V].u.out_kernelbuf.out_mmap_size +      
      config.bufconfig[CX88_VIDEO_CHANNEL_VBI].u.out_kernelbuf.out_mmap_size*/, 
         PROT_READ, MAP_SHARED, fd, 0);
   if (buf == MAP_FAILED)
   {
      perror("Error mmap'ing device:");
      return 0;
   }
   tune(tuner_devnode, fd, 400, buf, fmt, config);
   if (buf != MAP_FAILED)
   {
      munmap(buf, 
         config.bufconfig[CX88_VIDEO_CHANNEL_Y].u.out_kernelbuf.out_mmap_size /*+ 
         config.bufconfig[CX88_VIDEO_CHANNEL_U].u.out_kernelbuf.out_mmap_size +
         config.bufconfig[CX88_VIDEO_CHANNEL_V].u.out_kernelbuf.out_mmap_size +      
         config.bufconfig[CX88_VIDEO_CHANNEL_VBI].u.out_kernelbuf.out_mmap_size*/);
   }
   close(fd);
   return 0;
}

==> cx88-1.1.0.tar.gz: cx88-1.1.0/tests/hd3k_analog_kbuf.cpp <==
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/errno.h>
#include "cx88_video_ioctl.h"
#include "tuner_devnode_device.h"
#include "dtt7612.h"
#include "tda9887.h"

void capture(int fd, void *bufspace, cx88_video_format &fmt, cx88_video_config &config)
{
   cx88_video_capture capture;
   capture.in_input_source = CX88_VIDEO_INPUT_MUX1;
   capture.in_input_type = CX88_VIDEO_INPUT_COMPOSITE;
   capture.in_input_options = 0;
   capture.in_lock_timeout_ms = 1000;
   capture.in_lock_delay_ms = 100;
   capture.in_no_frame_intrs = 0;
   printf("Beginning capture\n");
   int error = ioctl(fd, CX88_VIDEO_CAPTURE, &capture);
   if (error)
   {
      printf("Error during capture ioctl\n");
      return;
   }
   char filename[32];
   for (int i = 0; i < config.in_num_frames; ++i)
   {
      cx88_video_wait wait;
      wait.in_timeout_ms = 0;
      if ((i == 0) || (wait.channels[CX88_VIDEO_CHANNEL_Y].out_even_frame_available <= i))
      {
         wait.in_channel_mask = CX88_VIDEO_FIELD_EVEN(CX88_VIDEO_CHANNEL_Y);
         error = ioctl(fd, CX88_VIDEO_WAIT, &wait);
         if (error)
         {
            printf("Error during wait ioctl\n");
            ioctl(fd, CX88_VIDEO_STOP);
            return;
         }
         if ((wait.out_status & CX88_VIDEO_BUS_ERROR) == CX88_VIDEO_BUS_ERROR)
         {
            printf("Fatal error: PCI bus error\n");
         }
         if ((wait.out_status & CX88_VIDEO_SYNC_ERROR) == CX88_VIDEO_SYNC_ERROR)
         {
            printf("Fatal error[%d]: DMA sync error\n", i);  
         }
         if ((wait.out_status & CX88_VIDEO_DMA_PROGRAM) == CX88_VIDEO_DMA_PROGRAM)
         {
            printf("Fatal error[%d]: DMA program error\n", i);
         }
         if ((wait.out_status & CX88_VIDEO_WAIT_ABORTED) == CX88_VIDEO_WAIT_ABORTED)
         {
            printf("Fatal error: Wait aborted\n");
         }
         if ((wait.out_status & CX88_VIDEO_CAP_TIMEOUT) == CX88_VIDEO_CAP_TIMEOUT)
         {
            printf("Nonfatal error: Capture timed out\n");
         }
         if (wait.out_status & CX88_VIDEO_CAP_FATAL)
         {
            ioctl(fd, CX88_VIDEO_STOP);
            return;
         }
      }
      /*if ((wait.channels[CX88_VIDEO_CHANNEL_Y].out_even_frame_available <= i) ||
          (wait.channels[CX88_VIDEO_CHANNEL_Y].out_odd_frame_available <= i))
      {
         printf("No frame available %d %d\n",
            wait.channels[CX88_VIDEO_CHANNEL_Y].out_even_frame_available,
            wait.channels[CX88_VIDEO_CHANNEL_Y].out_odd_frame_available);
         ioctl(fd, CX88_VIDEO_STOP);
         return;
      }*/
      printf("Y frames: %d, %d\n", 
         wait.channels[CX88_VIDEO_CHANNEL_Y].out_odd_frame_available,
         wait.channels[CX88_VIDEO_CHANNEL_Y].out_even_frame_available);
      printf("VBI frames: %d, %d\n",
         wait.channels[CX88_VIDEO_CHANNEL_VBI].out_odd_frame_available,
         wait.channels[CX88_VIDEO_CHANNEL_VBI].out_even_frame_available);
      printf("U frames: %d, %d\n",
         wait.channels[CX88_VIDEO_CHANNEL_U].out_odd_frame_available,
         wait.channels[CX88_VIDEO_CHANNEL_U].out_even_frame_available);
      printf("V frames: %d, %d\n",
         wait.channels[CX88_VIDEO_CHANNEL_V].out_odd_frame_available,
         wait.channels[CX88_VIDEO_CHANNEL_V].out_even_frame_available);
      sprintf(filename, "out%d.ppm", i);
      FILE *outfile = fopen(filename, "w");
      if (outfile == NULL)
      {
         printf("Error opening video output file\n");
         ioctl(fd, CX88_VIDEO_STOP);
         return;  
      }
      size_t linesize = fmt.out_geoms[CX88_VIDEO_CHANNEL_Y].line_size + config.bufconfig[CX88_VIDEO_CHANNEL_Y].in_x_header + config.bufconfig[CX88_VIDEO_CHANNEL_Y].in_x_trailer;
      size_t numlines = fmt.out_geoms[CX88_VIDEO_CHANNEL_Y].num_lines +  config.bufconfig[CX88_VIDEO_CHANNEL_Y].in_y_header + config.bufconfig[CX88_VIDEO_CHANNEL_Y].in_y_trailer;
      fprintf(outfile, "P6\n%ld %ld\n255\n", linesize / 3, numlines);
      size_t bufsize = linesize * numlines;
      uint8_t *ybuf = reinterpret_cast <uint8_t*> (bufspace) + 
         (i * config.bufconfig[CX88_VIDEO_CHANNEL_Y].u.out_kernelbuf.out_buffer_size);
      uint8_t *ubuf = reinterpret_cast <uint8_t*> (bufspace) + 
         config.bufconfig[CX88_VIDEO_CHANNEL_Y].u.out_kernelbuf.out_mmap_size +
         (i * config.bufconfig[CX88_VIDEO_CHANNEL_U].u.out_kernelbuf.out_buffer_size);
      uint8_t *vbuf = reinterpret_cast <uint8_t*> (bufspace) +
         config.bufconfig[CX88_VIDEO_CHANNEL_Y].u.out_kernelbuf.out_mmap_size +
         config.bufconfig[CX88_VIDEO_CHANNEL_U].u.out_kernelbuf.out_mmap_size +
         (i * config.bufconfig[CX88_VIDEO_CHANNEL_V].u.out_kernelbuf.out_buffer_size);
      /*for (size_t j = 0; j < bufsize; ++j)
      {
         uint8_t pixel[3];
         int8_t u = ubuf[j / 4] - 128;
         int8_t v = vbuf[j / 4] - 128;
         double r = 1.370705 * v + ybuf[j];
         double g = (double)(ybuf[j]) - (0.337633 * u) - (0.698001 * v);
         double b = 1.732446 * u + ybuf[j];
         if (r < 0) r = 0;
         if (g < 0) g = 0;
         if (b < 0) b = 0;
         if (r > 255) r = 255;
         if (g > 255) g = 255;
         if (b > 255) b = 255;
         pixel[0] = (uint8_t) r;
         pixel[1] = (uint8_t) g;
         pixel[2] = (uint8_t) b;
         fwrite(pixel, 3, 1, outfile);
      }*/
      for (size_t j = 0; j < bufsize; j += 3)
      {
         uint8_t pixel[3];
         pixel[0] = ybuf[j + 2];
         pixel[1] = ybuf[j + 1];
         pixel[2] = ybuf[j];
         fwrite(pixel, 3, 1, outfile);
      }
      fclose(outfile);
      if (fmt.in_enable_vbi)
      {
         if ((wait.channels[CX88_VIDEO_CHANNEL_VBI].out_even_frame_available <= i) ||
            (wait.channels[CX88_VIDEO_CHANNEL_VBI].out_odd_frame_available <= i))
         {
            printf("Warning: no VBI frame available\n");
            continue;
         }
         char *vbibuf = reinterpret_cast <char*> (bufspace) + 
            config.bufconfig[CX88_VIDEO_CHANNEL_Y].u.out_kernelbuf.out_mmap_size + 
            config.bufconfig[CX88_VIDEO_CHANNEL_U].u.out_kernelbuf.out_mmap_size +
            config.bufconfig[CX88_VIDEO_CHANNEL_V].u.out_kernelbuf.out_mmap_size +
            (i * config.bufconfig[CX88_VIDEO_CHANNEL_VBI].u.out_kernelbuf.out_buffer_size);
         bufsize = fmt.out_geoms[CX88_VIDEO_CHANNEL_VBI].line_size * fmt.out_geoms[CX88_VIDEO_CHANNEL_VBI].num_lines;
         sprintf(filename, "out%d.vbi", i);
         FILE *vbifile = fopen(filename, "w");
         if (vbifile == NULL)
         {
            printf("Warning: unable to open VBI output file\n");
            continue;
         }
         fwrite(vbibuf, bufsize, 1, vbifile);
         fclose(vbifile);
      }
   }
   ioctl(fd, CX88_VIDEO_STOP);
}

void tune(char *tuner_devnode, int fd, int buffers, void *bufspace, cx88_video_format &fmt, cx88_video_config &config)
{
   int error = 0;
   avb_channel channel;
   channel.video_format = AVB_VIDEO_FMT_NTSC_M;
   channel.audio_format = AVB_AUDIO_FMT_AUTO;
   channel.frequency_hz = 177000000;
   channel.bandwidth_hz = 6000000;
   tuner_config conf;
   error = conf.load_file("tuner_config");
   if (error)
   {
      printf("Error loading tuner config file\n");
      return;
   }
   tuner_devnode_device tuner_dev(conf, tuner_devnode, error);
   if (error)
   {
      printf("Error creating tuner devnode\n");
      return;
   }
   dtt7612 tuner_driver(conf, tuner_dev);
   error = tuner_driver.set_channel(channel);
   if (error)
   {
      printf("Error setting channel on tuner driver\n");
      return;  
   }
   printf("Ready to start tuner\n");
   error = tuner_driver.start(10000);
   if (error)
   {
      printf("Error starting tuner\n");
      return;
   }
   sleep(1);
   capture(fd, bufspace, fmt, config);
}

int main(void)
{
   cx88_video_format fmt;
   fmt.in_video_format = CX88_VIDEO_FMT_NTSCM;
   fmt.in_pixel_format = CX88_VIDEO_BGR24;
   fmt.in_rows = 480;
   fmt.in_cols = 720;
   fmt.in_x_offset = 0;
   fmt.in_x_size = 720;
   fmt.in_y_offset = 0;
   fmt.in_y_size = 480;
   fmt.in_enable_vbi = 0;
   fmt.in_interlace = 1;
   fmt.in_even_field.swap = 0;
   fmt.in_even_field.enable = 1;
   fmt.in_odd_field.swap = 0;
   fmt.in_odd_field.enable = 1;
   cx88_video_config config;
   uint32_t id;
   config.in_num_frames = 7;
   config.in_one_shot = 0;
   config.bufconfig[CX88_VIDEO_CHANNEL_Y].in_x_header = 120;
   config.bufconfig[CX88_VIDEO_CHANNEL_Y].in_x_trailer = 240;
   config.bufconfig[CX88_VIDEO_CHANNEL_Y].in_y_header = 35;
   config.bufconfig[CX88_VIDEO_CHANNEL_Y].in_y_trailer = 25;
   config.bufconfig[CX88_VIDEO_CHANNEL_Y].in_buftype = CX88_VIDEO_BUF_KERNEL;
   config.bufconfig[CX88_VIDEO_CHANNEL_U].in_buftype = CX88_VIDEO_BUF_KERNEL;
   config.bufconfig[CX88_VIDEO_CHANNEL_V].in_buftype = CX88_VIDEO_BUF_KERNEL;
   config.bufconfig[CX88_VIDEO_CHANNEL_VBI].in_buftype = CX88_VIDEO_BUF_KERNEL;
   int fd = open("/dev/cx88video0", O_RDWR);
   int error = ioctl(fd, CX88_COMMON_CARD_ID, &id);
   if (error)
   {
      printf("Error during card ID ioctl\n");
      return 1;
   }
   printf("Card ID = 0x%x\n", id);
   error = ioctl(fd, CX88_VIDEO_FORMAT, &fmt);
   if (error)
   {  
      printf("Error during format ioctl: %s\n", strerror(error));
      return 1;
   }
   cx88_video_clip clip;
   bzero(&clip, sizeof(clip));
   uint32_t pattern_size = sizeof(cx88_video_clip_pattern) + (2 * sizeof(uint16_t));
   cx88_video_clip_pattern *pattern = (cx88_video_clip_pattern*)(malloc((2 * pattern_size) + sizeof(uint16_t)));
   pattern->in_start_line = 100;
   pattern->in_end_line = 240;
   pattern->in_clip_type = CX88_CLIP_INCLUDE;
   pattern->in_num_points = 2;
   pattern->in_points[0] = 150;
   pattern->in_points[1] = 460;
   cx88_video_clip_pattern *pattern2 = (cx88_video_clip_pattern*) ((uint8_t*)pattern + pattern_size);
   pattern2->in_start_line = 260;
   pattern2->in_end_line = 310;
   pattern2->in_clip_type = CX88_CLIP_INCLUDE;
   pattern2->in_num_points = 3;
   pattern2->in_points[0] = 350;
   pattern2->in_points[1] = 560;
   pattern2->in_points[2] = 840;
   if (pattern == NULL)
   {
      printf("Unable to allocate clipping pattern buffer\n");
      return 1;
   }
   clip.channels[CX88_VIDEO_CHANNEL_Y].in_patterns = pattern;
   clip.channels[CX88_VIDEO_CHANNEL_Y].in_pattern_size = 2 * pattern_size + sizeof(uint16_t);
   error = ioctl(fd, CX88_VIDEO_CLIP, &clip);
   if (error)
   {
      printf("Error during clip ioctl: %s\n", strerror(errno));
      return 1;
   }
   printf("Y Geometry: %d x %d\n", fmt.out_geoms[CX88_VIDEO_CHANNEL_Y].line_size, fmt.out_geoms[CX88_VIDEO_CHANNEL_Y].num_lines);
   printf("U Geometry: %d x %d\n", fmt.out_geoms[CX88_VIDEO_CHANNEL_U].line_size, fmt.out_geoms[CX88_VIDEO_CHANNEL_U].num_lines);
   printf("V Geometry: %d x %d\n", fmt.out_geoms[CX88_VIDEO_CHANNEL_V].line_size, fmt.out_geoms[CX88_VIDEO_CHANNEL_V].num_lines);
   printf("VBI Geometry: %d x %d\n", fmt.out_geoms[CX88_VIDEO_CHANNEL_VBI].line_size, fmt.out_geoms[CX88_VIDEO_CHANNEL_VBI].num_lines);
   error = ioctl(fd, CX88_VIDEO_CONFIG, &config);
   if (error)
   {
      printf("Error during config ioctl: %s\n", strerror(errno));
      return 1;
   }   
   cx88_common_i2c_config i2config;
   i2config.in_i2c_addrs[0] = 0x61;
   i2config.in_num_i2c_devices = 1;
   error = ioctl(fd, CX88_COMMON_I2C_CONFIG, &i2config);
   if (error)
   {
      printf("Error during I2C config ioctl\n");
      return 1;  
   }
   printf("Tuner: %s\n", i2config.out_i2c_devnodes[0]);
   char tuner_devnode[CX88_COMMON_MAX_DEVNODE_NAME + 5];
   sprintf(tuner_devnode, "/dev/%s", i2config.out_i2c_devnodes[0]);
   void *buf = mmap(NULL,
      config.bufconfig[CX88_VIDEO_CHANNEL_Y].u.out_kernelbuf.out_mmap_size /*+ 
      config.bufconfig[CX88_VIDEO_CHANNEL_U].u.out_kernelbuf.out_mmap_size +
      config.bufconfig[CX88_VIDEO_CHANNEL_V].u.out_kernelbuf.out_mmap_size +      
      config.bufconfig[CX88_VIDEO_CHANNEL_VBI].u.out_kernelbuf.out_mmap_size*/, 
         PROT_READ, MAP_SHARED, fd, 0);
   if (buf == MAP_FAILED)
   {
      perror("Error mmap'ing device:");
      return 0;
   }
   tune(tuner_devnode, fd, 400, buf, fmt, config);
   if (buf != MAP_FAILED)
   {
      munmap(buf, 
         config.bufconfig[CX88_VIDEO_CHANNEL_Y].u.out_kernelbuf.out_mmap_size /*+ 
         config.bufconfig[CX88_VIDEO_CHANNEL_U].u.out_kernelbuf.out_mmap_size +
         config.bufconfig[CX88_VIDEO_CHANNEL_V].u.out_kernelbuf.out_mmap_size +      
         config.bufconfig[CX88_VIDEO_CHANNEL_VBI].u.out_kernelbuf.out_mmap_size*/);
   }
   close(fd);
   return 0;
}
